﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Globalization;

namespace klientMesic
{
    /// <summary>
    /// Simuluje cele pristani
    /// </summary>
    class Pristani
    {
        public int ID;
        public float x, y, vx, vy, p, l, r;
        public string url;
        public bool konec = false;

        private int pocetRadku = 5;
        private float meritko = 2F;
        private int velikostModulu = 20;
        private int vyska = 200;

        private string start = "/start";
        public string stav;

        public bool motorDoleva = false;
        public bool motorNahoru = false;
        public bool motorDoprava = false;

        private PictureBox canvas;
        public bool jeAutoPilot;

        public Pristani(string adresa, PictureBox canvas, bool auto)
        {
            this.url = adresa;
            this.canvas = canvas;
            this.jeAutoPilot = auto;

            Precti(start);
            canvas.Refresh();

            this.stav = "/stav?id=" + ID;
        }

        /// <summary>
        /// Cte http get requesty ze serveru
        /// </summary>
        /// <param name="plusUrl">URL navic pro start nebo zjisteni stavu hry</param>
        public void Precti(string plusUrl)
        {
            if (!konec)
            {
                HttpWebRequest req = null;
                try
                {
                    req = WebRequest.CreateHttp(this.url + plusUrl);
                }
                catch (Exception e)
                {
                    try
                    {
                        req = WebRequest.CreateHttp("http://"+this.url + plusUrl);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show("Špatná URL: " + ex.Message + (this.url + plusUrl));
                    }
                }


                HttpWebResponse res = (HttpWebResponse)req.GetResponse();
                using (StreamReader sr = new StreamReader(res.GetResponseStream()))
                {
                    for (int i = 0; i < pocetRadku; i++)
                    {
                        string radek = sr.ReadLine();

                        if (radek == "Zdar!" || radek == "Krach!")
                        {
                            konec = true;
                            if (radek == "Zdar!")
                            {
                                MessageBox.Show("Přistání bylo úspěšné", radek);
                            }
                            else
                            {
                                MessageBox.Show("Přistání bylo neúspěšné", radek);
                            }
                            break;
                        }

                        switch (i)
                        {
                            case 0:
                                this.ID = int.Parse(radek);
                                break;
                            case 1:
                                this.x = float.Parse(radek.Split(' ')[0], CultureInfo.InvariantCulture.NumberFormat);
                                this.y = float.Parse(radek.Split(' ')[1], CultureInfo.InvariantCulture.NumberFormat);
                                break;
                            case 2:
                                this.vx = float.Parse(radek.Split(' ')[0], CultureInfo.InvariantCulture.NumberFormat);
                                this.vy = float.Parse(radek.Split(' ')[1], CultureInfo.InvariantCulture.NumberFormat);
                                break;
                            case 3:
                                this.p = float.Parse(radek, CultureInfo.InvariantCulture.NumberFormat);
                                break;
                            case 4:
                                this.l = float.Parse(radek.Split(' ')[0], CultureInfo.InvariantCulture.NumberFormat);
                                this.r = float.Parse(radek.Split(' ')[1], CultureInfo.InvariantCulture.NumberFormat);
                                break;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Odeslani vypnuti nebo zapnuti motoru na server
        /// </summary>
        /// <param name="arg">Argument</param>
        public void Posli(string arg)
        {
            Precti(stav + "&" + arg);
        }

        /// <summary>
        /// Vykresluje hru
        /// </summary>
        /// <param name="g">Graphics canvasu</param>
        public void Vykresli(Graphics g)
        {
            try
            {
                #region Modul
                Rectangle modulRect = new Rectangle((int)(x * meritko), (int)((vyska - y) * meritko), (int)(velikostModulu * meritko), (int)(velikostModulu * meritko));
                //g.FillRectangle(Brushes.Red, modulRect);

                Image modul = Image.FromFile("modul.png");
                g.DrawImage(modul, modulRect);
                #endregion

                #region Pristavaci plocha
                Rectangle plocha = new Rectangle((int)(l * meritko), (int)((vyska * meritko) + (velikostModulu / 2 * meritko)), (int)((r - l) * meritko), (int)(2 * meritko));
                g.FillRectangle(Brushes.Black, plocha);
                #endregion
            }
            catch (Exception e)
            {

            }
        }

        /// <summary>
        /// Zobrazuje stav hry
        /// </summary>
        /// <param name="doleva">Levy motor kontrolka</param>
        /// <param name="nahoru">Horni motor kontrolka</param>
        /// <param name="doprava">Pravy motor kontrolka</param>
        /// <param name="palivoBar">Kontrolka paliva</param>
        public void ZobrazStav(PictureBox doleva, PictureBox nahoru, PictureBox doprava, ProgressBar palivoBar, Label vxLbl, Label vyLbl)
        {
            doleva.BackColor = motorDoleva ? Color.Green : Color.Red;
            nahoru.BackColor = motorNahoru ? Color.Green : Color.Red;
            doprava.BackColor = motorDoprava ? Color.Green : Color.Red;
            palivoBar.Value = (int)p;
            vxLbl.Text = String.Format("{0:0.00}", vx);
            vyLbl.Text = String.Format("{0:0.00}", vy);
        }

        /// <summary>
        /// Kontroluje, jestli pristani bylo uspesne
        /// </summary>
        /// <returns>true pokud ano; jinak false</returns>
        public bool UspesnePristani()
        {
            if (y == 10 && (x - 10 >= l && x + 10 <= r) && (Math.Abs(vx) <= 5 && Math.Abs(vy) <= 5)) //TODO hlidat rychlost
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// Kontroluje, jak se hybe
        /// </summary>
        /// <param name="T">casovy usek bez zmeny</param>
        public void Pohyb()
        {
            Precti(stav);
        }

        private void NoveHodnoty(float T, out float noveX, out float noveY, out float noveVx, out float noveVy, out float noveP)
        {
            int doprava = motorDoprava ? 1 : 0;
            int doleva = motorDoleva ? 1 : 0;
            int nahoru = motorNahoru ? 1 : 0;

            noveX = (float)(x + vx * T + 0.5 * (6 * doprava - 6 * doleva) * Math.Pow(T, 2));
            noveY = (float)(y + vy * T + 0.5 * (-7 + 12 * nahoru) * Math.Pow(T, 2));
            noveVx = (float)(vx + (6 * doprava - 6 * doleva) * T);
            noveVy = (float)(vy + (-7 + 12 * y) * T);
            noveP = (float)(p - (0.5 * doleva + 0.5 * doprava + nahoru) * T);

            this.x = noveX;
            this.y = noveY;
            this.vx = noveVx;
            this.vy = noveVy;
            this.p = noveP;
        }

        public void AutoPilot()
        {
            if (vy < -15)
            {
                Posli("nahoru=1");
                motorNahoru = true;
            }
            if (vy > -2)
            {
                Posli("nahoru=0");
                motorNahoru = false;
            }
            if (y < 50)
            {
                if (vy < -5)
                {
                    Posli("nahoru=1");
                    motorNahoru = true;
                }
                if (vy > -2)
                {
                    Posli("nahoru=0");
                    motorNahoru = false;
                }
            }

            /*if ((x+10) > r)
            {
                Posli("doleva=1");
                motorDoleva = true;
                Posli("doprava=0");
                motorDoprava = false;
            }
            if ((x - 10) < l)
            {
                Posli("doleva=0");
                motorDoleva = false;
                Posli("doprava=1");
                motorDoprava = true;
            }*/
        }

        public string TestVypis()
        {
            return String.Format("\nID:{0}\nX:{1} Y:{2}\nVx: {3} Vy: {4}\n P: {5}\n L: {6} R: {7}", ID, x, y, vx, vy, p, l, r);
        }

    }
}
